@@ -27,8 +27,7 @@ import Control.Monad.State.Strict
 import Data.Attoparsec.ByteString.Char8
 import Data.Bits
 import qualified Data.ByteString.Char8 as BS
-import Data.Either
-import Data.Either.Combinators
+import Data.Either.Combinators as EC
 import Data.List
 import qualified Data.Map as M
 import Data.Tree
@@ -92,7 +91,7 @@ xform = map buildPython . dependencyOrder
     processXHeader :: XHeader
                    -> State TypeInfoMap (String, Suite ())
     processXHeader header = do
-      let imports = [mkImport "xcffib", mkImport "struct", mkImport "six"]
+      let imports = [mkImport "xcffib", mkImport "struct", mkImport "io"]
           version = mkVersion header
           key = maybeToList $ mkKey header
           globals = [mkDict "_events", mkDict "_errors"]
@@ -213,7 +212,7 @@ xExpressionToPyExpr _ (Bit i) = BinaryOp (ShiftLeft ()) (mkInt 1) (mkInt i) ()
 xExpressionToPyExpr acc (FieldRef n) = mkName $ acc n
 xExpressionToPyExpr _ (EnumRef (UnQualType enum) n) = mkName $ enum ++ "." ++ n
 -- Currently xcb only uses unqualified types, not sure how qualtype should behave
-xExpressionToPyExpr _ (EnumRef (QualType _ _) _) = error "Qualified type, unknown behavior"
+xExpressionToPyExpr _ (EnumRef (QualType ext n) _) = mkName $ ext ++ "." ++ n
 xExpressionToPyExpr acc (PopCount e) =
   mkCall "xcffib.popcount" [xExpressionToPyExpr acc e]
 -- http://cgit.freedesktop.org/xcb/proto/tree/doc/xml-xcb.txt#n290
@@ -227,7 +226,10 @@ xExpressionToPyExpr acc (Unop o e) =
   let o' = xUnopToPyOp o
       e' = xExpressionToNestedPyExpr acc e
   in Paren (UnaryOp o' e' ()) ()
-xExpressionToPyExpr _ (ParamRef n) = mkName n
+xExpressionToPyExpr acc (ParamRef n) =
+    if n == "num_axes"
+    then mkName $ acc n
+    else error ("unsupported paramref " ++ n)
 
 getConst :: XExpression -> Maybe Int
 getConst (Value i) = Just i
@@ -288,12 +290,14 @@ structElemToPyUnpack :: Expr ()
                      -> Either (Maybe String, String)
                                (String, Either (Expr (), Expr ())
                                                ([(Expr (), [GenStructElem Type])]), Maybe Int)
-structElemToPyUnpack _ _ _ (Pad i) = Left (Nothing, mkPad i)
+structElemToPyUnpack _ _ _ (Pad PadBytes i) = Left (Nothing, mkPad i)
+structElemToPyUnpack _ _ _ (Pad PadAlignment _) = Left (Nothing, "")
 
 -- XXX: This is a cheap hack for noop, we should really do better.
 structElemToPyUnpack _ _ _ (Doc _ _ _) = Left (Nothing, "")
 -- XXX: What does fd mean? we should implement it correctly
 structElemToPyUnpack _ _ _ (Fd _) = Left (Nothing, "")
+structElemToPyUnpack _ _ _ (Length _ _) = Left (Nothing, "")
 
 -- The switch fields pick the way to expression to pack based on the expression
 structElemToPyUnpack _ _ _ (Switch name expr _ bitcases) =
@@ -321,6 +325,9 @@ structElemToPyUnpack unpacker ext m (X.List n typ len _) =
       cons = case m M.! typ of
                BaseType c -> mkStr c
                CompositeType tExt c | ext /= tExt -> mkName $ tExt ++ "." ++ c
+               CompositeType _ "DeviceTimeCoord" ->
+                 let wrapper = mkName "xcffib.__DeviceTimeCoord_wrapper"
+                 in mkCall wrapper [mkName "DeviceTimeCoord", mkName (attr "num_axes")]
                CompositeType _ c -> mkName c
       list = mkCall "xcffib.List" [ unpacker
                                   , cons
@@ -351,13 +358,14 @@ structElemToPyPack :: String
                    -> Either (Maybe String, String) [(String, Either (Maybe (Expr ()))
                                                                      [(Expr (), [GenStructElem Type])]
                                                     )]
-structElemToPyPack _ _ _ (Pad i) = Left (Nothing, mkPad i)
--- TODO: implement doc and fd?
+structElemToPyPack _ _ _ (Pad _ i) = Left (Nothing, mkPad i)
+-- TODO: implement these?
 structElemToPyPack _ _ _ (Doc _ _ _) = Left (Nothing, "")
 structElemToPyPack _ _ _ (Fd _) = Left (Nothing, "")
+structElemToPyPack _ _ _ (Length _ _) = Left (Nothing, "")
 structElemToPyPack _ _ accessor (Switch n expr _ bitcases) =
   let name = accessor n
-      cmp = xExpressionToPyExpr id expr
+      cmp = xExpressionToPyExpr accessor expr
       elems = map (mkSwitch cmp) bitcases
   in Right $ [(name, Right elems)]
     where
@@ -365,17 +373,18 @@ structElemToPyPack _ _ accessor (Switch n expr _ bitcases) =
                -> BitCase
                -> (Expr (), [GenStructElem Type])
       mkSwitch cmp (BitCase _ bcCmp _ elems') =
-        let cmpVal = xExpressionToPyExpr id bcCmp
+        let cmpVal = xExpressionToPyExpr accessor bcCmp
             equality = BinaryOp (P.BinaryAnd ()) cmp cmpVal ()
         in (equality, elems')
-structElemToPyPack _ m accessor (SField n typ _ _) =
+structElemToPyPack ext m accessor (SField n typ _ _) =
   let name = accessor n
   in case m M.! typ of
        BaseType c -> Left (Just name, c)
-       CompositeType _ typNam ->
+       CompositeType tExt typNam ->
          let cond = mkCall "hasattr" [mkArg name, ArgExpr (mkStr "pack") ()]
              trueB = mkCall (name ++ ".pack") noArgs
-             synthetic = mkCall (typNam ++ ".synthetic") [mkArg ("*" ++ name)]
+             typNam' = if ext == tExt then typNam else tExt ++ "." ++ typNam
+             synthetic = mkCall (typNam' ++ ".synthetic") [mkArg ("*" ++ name)]
              falseB = mkCall (mkDot synthetic "pack") noArgs
          in Right $ [(name
                     , Left (Just (CondExpr trueB cond falseB ()))
@@ -428,7 +437,7 @@ structElemToPyPack _ m accessor (ValueParam typ mask _ list) =
       "ValueParams other than CARD{16,32} not allowed.")
 
 buf :: Suite ()
-buf = [mkAssign "buf" (mkCall "six.BytesIO" noArgs)]
+buf = [mkAssign "buf" (mkCall "io.BytesIO" noArgs)]
 
 mkPackStmts :: String
             -> String
@@ -439,8 +448,9 @@ mkPackStmts :: String
             -> ([String], Suite ())
 mkPackStmts ext name m accessor prefix membs =
   let packF = structElemToPyPack ext m accessor
-      (toPack, stmts) = partitionEithers $ map packF membs
-      (args, keys) = let (as, ks) = unzip toPack in (catMaybes as, ks)
+      (toPack, stmts) = span EC.isLeft $ map packF membs
+      stmts' = map (either mkBasePack id) stmts
+      (args, keys) = let (as, ks) = unzip (map EC.fromLeft' toPack) in (catMaybes as, ks)
 
       -- In some cases (e.g. xproto.ConfigureWindow) there is padding after
       -- value_mask. The way the xml specification deals with this is by
@@ -448,7 +458,7 @@ mkPackStmts ext name m accessor prefix membs =
       -- implying it implicitly. Thus, we want to make sure that if we've already
       -- been told to pack something explcitly, that we don't also pack it
       -- implicitly.
-      (listNames, listOrSwitches) = unzip $ filter (flip notElem args . fst) (concat stmts)
+      (listNames, listOrSwitches) = unzip $ filter (flip notElem args . fst) (concat stmts')
       listWrites = concat $ map (uncurry mkWrites) $ zip listNames listOrSwitches
       listNames' = case (ext, name) of
                      -- XXX: QueryTextExtents has a field named "odd_length"
@@ -482,7 +492,14 @@ mkPackStmts ext name m accessor prefix membs =
       mkPop :: String
             -> String
             -> Statement ()
-      mkPop toPop n = mkAssign n $ mkCall (mkDot toPop "pop") [mkInt 0]
+      mkPop toPop n =
+        let pop = mkCall (mkDot toPop "pop") [mkInt 0]
+        in if null n then StmtExpr pop () else mkAssign n pop
+
+      mkBasePack (Nothing, "") = []
+      mkBasePack (n, c) =
+        let n' = maybe "" id n
+        in [(n', Left (Just (mkCall "struct.pack" [mkStr ('=' : c), mkName n'])))]
 
 mkPackMethod :: String
              -> String
@@ -638,7 +655,7 @@ mkSyntheticMethod membs = do
   if null names then [] else [Decorated [classmethod] synthetic ()]
     where
       getName :: GenStructElem Type -> Maybe String
-      getName (Pad _) = Nothing
+      getName (Pad _ _) = Nothing
       getName (X.List n _ _ _) = Just n
       getName (SField n _ _ _) = Just n
       getName (ExprField n _ _) = Just n
@@ -646,6 +663,7 @@ mkSyntheticMethod membs = do
       getName (Switch n _ _ _) = Just n
       getName (Doc _ _ _) = Nothing
       getName (Fd n) = Just n
+      getName (Length _ _) = Nothing
 
       assign :: String -> Statement ()
       assign n = mkAssign (mkDot "self" n) $ mkName n
@@ -674,8 +692,8 @@ processXDecl ext (XStruct n _ membs) = do
         let rhs = mkInt theLen
         return $ mkAssign "fixed_size" rhs
   modify $ mkModify ext n (CompositeType ext n)
-  return $ Declaration [mkXClass n "xcffib.Struct" statements (pack : fixedLength ++ synthetic)]
-processXDecl ext (XEvent name opcode _ membs noSequence) = do
+  return $ Declaration [mkXClass n "xcffib.Struct" False statements (pack : fixedLength ++ synthetic)]
+processXDecl ext (XEvent name opcode _ xge membs noSequence) = do
   m <- get
   let cname = name ++ "Event"
       prefix = if fromMaybe False noSequence then "x" else "x%c2x"
@@ -683,7 +701,9 @@ processXDecl ext (XEvent name opcode _ membs noSequence) = do
       synthetic = mkSyntheticMethod membs
       (statements, _) = mkStructStyleUnpack prefix ext m membs
       eventsUpd = mkDictUpdate "_events" opcode cname
-  return $ Declaration [ mkXClass cname "xcffib.Event" statements (pack : synthetic)
+      isxge = fromMaybe False xge
+      -- xgeexp = mkAssign "xge" (if fromMaybe False xge then (mkName "True") else (mkName "False"))
+  return $ Declaration [ mkXClass cname "xcffib.Event" isxge statements (pack : synthetic)
                        , eventsUpd
                        ]
 processXDecl ext (XError name opcode _ membs) = do
@@ -694,7 +714,7 @@ processXDecl ext (XError name opcode _ membs) = do
       (statements, _) = mkStructStyleUnpack prefix ext m membs
       errorsUpd = mkDictUpdate "_errors" opcode cname
       alias = mkAssign ("Bad" ++ name) (mkName cname)
-  return $ Declaration [ mkXClass cname "xcffib.Error" statements [pack]
+  return $ Declaration [ mkXClass cname "xcffib.Error" False statements [pack]
                        , alias
                        , errorsUpd
                        ]
@@ -710,7 +730,7 @@ processXDecl ext (XRequest name opcode _ membs reply) = do
         GenXReply _ reply' <- reply
         let (replyStmts, _) = mkStructStyleUnpack "x%c2x4x" ext m reply'
             replyName = name ++ "Reply"
-            theReply = mkXClass replyName "xcffib.Reply" replyStmts []
+            theReply = mkXClass replyName "xcffib.Reply" False replyStmts []
             replyType = mkAssign "reply_type" $ mkName replyName
             cookie = mkClass cookieName "xcffib.Cookie" [replyType]
         return [theReply, cookie]
@@ -721,7 +741,7 @@ processXDecl ext (XRequest name opcode _ membs reply) = do
       isChecked = pyTruth $ isJust reply
       argChecked = ArgKeyword (ident "is_checked") (mkName "is_checked") ()
       checkedParam = Param (ident "is_checked") Nothing (Just isChecked) ()
-      allArgs = (mkParams $ "self" : args) ++ [checkedParam]
+      allArgs = (mkParams $ "self" : (filter (not . null) args)) ++ [checkedParam]
       mkArg' = flip ArgExpr ()
       ret = mkReturn $ mkCall "self.send_request" ((map mkArg' [ mkInt opcode
                                                                , mkName "buf"
@@ -734,16 +754,17 @@ processXDecl ext (XRequest name opcode _ membs reply) = do
 processXDecl ext (XUnion name _ membs) = do
   m <- get
   let unpackF = structElemToPyUnpack unpackerCopy ext m
-      (fields, listInfo) = partitionEithers $ map unpackF membs
-      toUnpack = concat $ map mkUnionUnpack fields
-      (names, listOrSwitches, _) = unzip3 listInfo
-      (exprs, _) = unzip $ map fromLeft' listOrSwitches
+      (fields, listInfo) = span EC.isLeft $ map unpackF membs
+      toUnpack = concat $ map (mkUnionUnpack . EC.fromLeft') fields
+      listInfo' = map (either mkBaseUnpack id) listInfo
+      (names, listOrSwitches, _) = unzip3 listInfo'
+      (exprs, _) = unzip $ map EC.fromLeft' listOrSwitches
       lists = map (uncurry mkAssign) $ zip (map mkAttr names) exprs
       initMethod = lists ++ toUnpack
       -- Here, we only want to pack the first member of the union, since every
       -- member is the same data and we don't want to repeatedly pack it.
       pack = mkPackMethod ext name m Nothing [head membs] Nothing
-      decl = [mkXClass name "xcffib.Union" initMethod [pack]]
+      decl = [mkXClass name "xcffib.Union" False initMethod [pack]]
   modify $ mkModify ext name (CompositeType ext name)
   return $ Declaration decl
   where
@@ -753,6 +774,8 @@ processXDecl ext (XUnion name _ membs) = do
     mkUnionUnpack (n, typ) =
       mkUnpackFrom unpackerCopy (maybeToList n) typ
 
+    mkBaseUnpack _ = error "xcffib: trailing base types unpack not implemented"
+
 processXDecl ext (XidUnion name _) =
   -- These are always unions of only XIDs.
   do modify $ mkModify ext name (BaseType "I")
@@ -771,7 +794,7 @@ processXDecl ext (XidUnion name _) =
 -- again.
 processXDecl ext (XEventStruct name _) = do
   modify $ mkModify ext name (CompositeType ext name)
-  return $ Declaration $ [mkXClass name "xcffib.Buffer" [] []]
+  return $ Declaration $ [mkXClass name "xcffib.Buffer" False [] []]
 
 mkVersion :: XHeader -> Suite ()
 mkVersion header =
