@@ -1,9 +1,10 @@
 import xcffib
 import struct
-import six
+import io
 _events = {}
 _errors = {}
 class KeymapNotifyEvent(xcffib.Event):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -13,7 +14,7 @@ class KeymapNotifyEvent(xcffib.Event):
         self.keys = xcffib.List(unpacker, "B", 31)
         self.bufsize = unpacker.offset - base
     def pack(self):
-        buf = six.BytesIO()
+        buf = io.BytesIO()
         buf.write(struct.pack("=B", 11))
         buf.write(xcffib.pack_list(self.keys, "B"))
         buf_len = len(buf.getvalue())
