@@ -1,9 +1,10 @@
 import xcffib
 import struct
-import six
+import io
 _events = {}
 _errors = {}
 class ClientMessageData(xcffib.Union):
+    xge = False
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
             unpacker = xcffib.MemoryUnpacker(unpacker.pack())
@@ -12,7 +13,7 @@ class ClientMessageData(xcffib.Union):
         self.data16 = xcffib.List(unpacker.copy(), "H", 10)
         self.data32 = xcffib.List(unpacker.copy(), "I", 5)
     def pack(self):
-        buf = six.BytesIO()
+        buf = io.BytesIO()
         buf.write(xcffib.pack_list(self.data8, "B"))
         return buf.getvalue()
 xcffib._add_ext(key, unionExtension, _events, _errors)
