@@ -22,10 +22,7 @@ import platform
 import struct
 import weakref
 
-try:
-    from xcffib._ffi import ffi
-except ImportError:
-    from xcffib.ffi_build import ffi
+from .ffi import ffi
 
 if platform.system() == "Darwin":
     soname = "libxcb.dylib"
@@ -37,8 +34,8 @@ else:
         soname = "libxcb.so"
 lib = ffi.dlopen(soname)
 
-__xcb_proto_version__ = "1.15.2"
-__version__ = "1.3.0"
+__xcb_proto_version__ = "1.16.0"
+__version__ = "1.5.0"
 
 X_PROTOCOL = lib.X_PROTOCOL
 X_PROTOCOL_REVISION = lib.X_PROTOCOL_REVISION
